@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.cpp,v 1.43 2004/02/13 16:55:59 marco Exp $
+ *  $Id: ContentHandler.cpp,v 1.47.4.2 2004/04/18 10:47:27 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,8 +34,7 @@
 #include "nsIURL.h"
 #include "nsILocalFile.h"
 #include "nsIMIMEInfo.h"
-
-#include "nsIWebNavigation.h" // Needed to create the LoadType flag
+#include "nsIInterfaceRequestorUtils.h"
 
 #include "ephy-prefs.h"
 #include "eel-gconf-extensions.h"
@@ -60,12 +59,12 @@ NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
 GContentHandler::GContentHandler() : mMimeType(nsnull)
 {
-	LOG ("GContentHandler ctor")
+	LOG ("GContentHandler ctor (%p)", this)
 }
 
 GContentHandler::~GContentHandler()
 {
-	LOG ("GContentHandler dtor")
+	LOG ("GContentHandler dtor (%p)", this)
 
 	nsMemory::Free (mMimeType);
 }
@@ -129,7 +128,7 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 					  _retval);
 	}
 
-	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_QueryInterface (aWindowContext);
+	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_GetInterface (aWindowContext);
 	GtkWidget *parentWindow = GTK_WIDGET (MozillaFindGtkParent (parentDOMWindow));
 
 	dialog = ephy_file_chooser_new (_("Save"), parentWindow,
@@ -139,7 +138,7 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 					   NS_ConvertUCS2toUTF8 (aDefaultFile).get());
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
-	if (response == EPHY_RESPONSE_SAVE)
+	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 
@@ -250,23 +249,26 @@ NS_METHOD GContentHandler::Init (void)
 	return NS_OK;
 }
 
-NS_METHOD GContentHandler::MIMEConfirmAction (PRBool autoDownload)
+NS_METHOD GContentHandler::MIMEConfirmAction ()
 {
 	GtkWidget *dialog;
 	GtkWidget *hbox, *vbox, *label, *image;
+	const char *action_label;
 	char *text;
 	int response;
 
-	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_QueryInterface (mContext);
+	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_GetInterface (mContext);
 	GtkWindow *parentWindow = GTK_WINDOW (MozillaFindGtkParent(parentDOMWindow));
 
+	action_label =  (mAction == CONTENT_ACTION_OPEN) ||
+			(mAction == CONTENT_ACTION_OPEN_TMP) ?
+			_("_Open") : _("_Download");
+
 	dialog = gtk_dialog_new_with_buttons
 		("", parentWindow, GTK_DIALOG_NO_SEPARATOR,
 		 _("_Save As..."), CONTENT_ACTION_SAVEAS,
 		 GTK_STOCK_CANCEL, CONTENT_ACTION_NONE,
-		 mAction == CONTENT_ACTION_OPEN ?
-		 _("_Open") : _("_Download"), mAction,
-		 NULL);
+		 action_label, mAction, NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), (guint)mAction);
 	
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
@@ -278,8 +280,6 @@ NS_METHOD GContentHandler::MIMEConfirmAction (PRBool autoDownload)
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
 			    TRUE, TRUE, 0);
 
-	g_print ("AAA %d %p", mPermission, mHelperApp);
-
 	if (mPermission == EPHY_MIME_PERMISSION_UNSAFE && mHelperApp)
 	{
 		text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
@@ -289,7 +289,7 @@ NS_METHOD GContentHandler::MIMEConfirmAction (PRBool autoDownload)
 					  "It's not safe to open it directly. You "
 					  "can save it instead."));
 	}
-	else if (mAction == CONTENT_ACTION_OPEN && !autoDownload)
+	else if (mAction == CONTENT_ACTION_OPEN_TMP)
 	{
 		text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
 					_("Open the file in another application?"),
@@ -317,6 +317,7 @@ NS_METHOD GContentHandler::MIMEConfirmAction (PRBool autoDownload)
 	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 
 	label = gtk_label_new (NULL);
+	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_label_set_markup (GTK_LABEL (label), text);
@@ -370,7 +371,7 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 
 	if (!auto_downloads || mAction == CONTENT_ACTION_DOWNLOAD)
 	{
-		MIMEConfirmAction (auto_downloads);
+		MIMEConfirmAction ();
 	}
 
 	nsCOMPtr<nsIMIMEInfo> mimeInfo;
