@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-persist.cpp,v 1.31 2004/02/29 13:25:19 chpe Exp $
+ *  $Id: mozilla-embed-persist.cpp,v 1.31.4.1 2004/05/03 19:07:06 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -226,12 +226,17 @@ impl_save (EphyEmbedPersist *persist)
 
 	/* Get the current page descriptor */
 	nsCOMPtr<nsISupports> pageDescriptor;
-	if (flags & EMBED_PERSIST_COPY_PAGE)
+	if (browser)
 	{
 	        browser->GetPageDescriptor(getter_AddRefs(pageDescriptor));
-		NS_ENSURE_TRUE (pageDescriptor, FALSE);
 	}
 
+	/* if we have COPY_PAGE, we *need* to have a page descriptor, else we'll re-fetch
+	 * the page, which will possibly give a different page than the original which we
+	 * need for view source
+	 */
+	NS_ENSURE_TRUE (!(flags & EMBED_PERSIST_COPY_PAGE) || pageDescriptor, FALSE);
+
 	if (filename == NULL)
 	{
 		/* Create an header sniffer and do the save */
@@ -245,7 +250,7 @@ impl_save (EphyEmbedPersist *persist)
 		if (!sniffer) return FALSE;
  
 		webPersist->SetProgressListener(sniffer);
-		rv = webPersist->SaveURI(inURI, nsnull, nsnull, nsnull, nsnull, tmpFile);
+		rv = webPersist->SaveURI(inURI, pageDescriptor, nsnull, nsnull, nsnull, tmpFile);
 		if (NS_FAILED (rv)) return FALSE;
 	}
 	else
