@@ -15,9 +15,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed.cpp,v 1.60 2004/02/12 20:18:52 chpe Exp $
+ *  $Id: mozilla-embed.cpp,v 1.62.4.4 2004/04/28 18:09:59 chpe Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "mozilla-embed.h"
 #include "mozilla-embed-event.h"
 #include "ephy-embed-shell.h"
@@ -44,7 +48,7 @@
 static void	mozilla_embed_class_init	(MozillaEmbedClass *klass);
 static void	mozilla_embed_init		(MozillaEmbed *gs);
 static void	mozilla_embed_destroy		(GtkObject *object);
-static void	ephy_embed_iface_init		(EphyEmbedIFace *iface);
+static void	ephy_embed_iface_init		(EphyEmbedIface *iface);
 
 static void mozilla_embed_connect_signals	(MozillaEmbed *membed);
 static void mozilla_embed_location_changed_cb	(GtkMozEmbed *embed,
@@ -110,7 +114,7 @@ impl_manager_can_do_command (EphyCommandManager *manager,
 }
 
 static void
-ephy_command_manager_iface_init (EphyCommandManagerIFace *iface)
+ephy_command_manager_iface_init (EphyCommandManagerIface *iface)
 {
 	iface->do_command = impl_manager_do_command;
 	iface->can_do_command = impl_manager_can_do_command;
@@ -481,8 +485,6 @@ impl_get_location (EphyEmbed *embed,
 	nsresult rv;
 	nsCAutoString url;
 
-	if (mpriv->browser == NULL) return NULL;
-	
 	if (toplevel)
 	{
 		rv = mpriv->browser->GetDocumentUrl (url);
@@ -832,9 +834,14 @@ mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	rv = ctx.GetKeyEventInfo (ev, info);
 	if (NS_FAILED (rv)) return ret;
 
-	if (info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
+	if ((info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
 	    (info->modifier == GDK_SHIFT_MASK ||
 	     info->modifier == GDK_CONTROL_MASK))
+#if MOZILLA_SNAPSHOT > 14
+	    || (info->keycode == nsIDOMKeyEvent::DOM_VK_CONTEXT_MENU &&
+		!info->modifier)
+#endif
+	   )
 	{
 		/* Translate relative coordinates to absolute values, and try
 		   to avoid covering links by adding a little offset. */
@@ -974,45 +981,41 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 static void
 mozilla_embed_new_window_cb (GtkMozEmbed *embed, 
 			     GtkMozEmbed **newEmbed,
-                             guint chromemask, 
+                             guint chrome_mask, 
 			     MozillaEmbed *membed)
 {
-	int i;
-	EmbedChromeMask mask = EMBED_CHROME_OPENASPOPUP;
+	guint i;
+	guint mask = 0;
 	EphyEmbed *new_embed = NULL;
 
 	struct
 	{
-		guint chromemask;
-		EmbedChromeMask embed_mask;
+		guint mozilla_mask;
+		guint embed_mask;
 	}
 	conversion_map [] =
 	{
-		{ GTK_MOZ_EMBED_FLAG_DEFAULTCHROME, EMBED_CHROME_DEFAULT },
-		{ GTK_MOZ_EMBED_FLAG_MENUBARON, EMBED_CHROME_MENUBARON },
-		{ GTK_MOZ_EMBED_FLAG_TOOLBARON, EMBED_CHROME_TOOLBARON },
-		{ GTK_MOZ_EMBED_FLAG_PERSONALTOOLBARON, EMBED_CHROME_BOOKMARKSBAR_DEFAULT },
-		{ GTK_MOZ_EMBED_FLAG_STATUSBARON, EMBED_CHROME_STATUSBARON },
-		{ GTK_MOZ_EMBED_FLAG_WINDOWRAISED, EMBED_CHROME_WINDOWRAISED },
-		{ GTK_MOZ_EMBED_FLAG_WINDOWLOWERED, EMBED_CHROME_WINDOWLOWERED },
-		{ GTK_MOZ_EMBED_FLAG_CENTERSCREEN, EMBED_CHROME_CENTERSCREEN },
-		{ 0, EMBED_CHROME_NONE }
+		{ GTK_MOZ_EMBED_FLAG_MENUBARON, EPHY_EMBED_CHROME_MENUBAR },
+		{ GTK_MOZ_EMBED_FLAG_TOOLBARON, EPHY_EMBED_CHROME_TOOLBAR },
+		{ GTK_MOZ_EMBED_FLAG_STATUSBARON, EPHY_EMBED_CHROME_STATUSBAR },
+		{ GTK_MOZ_EMBED_FLAG_PERSONALTOOLBARON, EPHY_EMBED_CHROME_BOOKMARKSBAR },
+		{ 0, 0 }
 	};
 
-	if (chromemask & GTK_MOZ_EMBED_FLAG_OPENASCHROME)
+	if (chrome_mask & GTK_MOZ_EMBED_FLAG_OPENASCHROME)
 	{
 		*newEmbed = _mozilla_embed_new_xul_dialog ();
 		return;
 	}
 
-	for (i = 0; conversion_map[i].chromemask != 0; i++)
+	for (i = 0; conversion_map[i].mozilla_mask != 0; i++)
 	{
-		if (chromemask & conversion_map[i].chromemask)
+		if (chrome_mask & conversion_map[i].mozilla_mask)
 		{
-			mask = (EmbedChromeMask) (mask | conversion_map[i].embed_mask);	
+			mask |= conversion_map[i].embed_mask;
 		}
 	}
-	
+
 	g_signal_emit_by_name (membed, "ge_new_window", &new_embed, mask);
 
 	g_assert (new_embed != NULL);
@@ -1068,7 +1071,7 @@ mozilla_embed_security_level (MozillaEmbed *membed)
 }
 
 static void
-ephy_embed_iface_init (EphyEmbedIFace *iface)
+ephy_embed_iface_init (EphyEmbedIface *iface)
 {
 	iface->load_url = impl_load_url; 
 	iface->stop_load = impl_stop_load;
