@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,12 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: print-dialog.c,v 1.21 2004/01/04 22:27:33 chpe Exp $
+ *  $Id: print-dialog.c,v 1.23.4.1 2004/03/29 16:14:07 federico Exp $
  */
 
 #include "print-dialog.h"
 #include "ephy-embed-dialog.h"
 #include "ephy-file-chooser.h"
+#include "ephy-file-helpers.h"
 #include "ephy-stock-icons.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-debug.h"
@@ -253,7 +254,7 @@ print_filechooser_response_cb (GtkDialog *fc,
 			       int response,
 			       EphyDialog *dialog)
 {
-	if (response == EPHY_RESPONSE_SAVE)
+	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		char *filename;
 
@@ -329,8 +330,11 @@ ephy_print_dialog_new (GtkWidget *parent,
 		g_object_set (G_OBJECT (dialog), "parent-window", parent, NULL);
 	}
 
-	ephy_dialog_construct (dialog,  print_props,
-			       "print.glade", "print_dialog");
+	ephy_dialog_construct (dialog, 
+			       print_props,
+			       ephy_file ("print.glade"),
+			       "print_dialog",
+			       NULL);
 
 	window = ephy_dialog_get_control (dialog, print_props[WINDOW_PROP].id);
 	icon = gtk_widget_render_icon (window, 
@@ -358,8 +362,11 @@ ephy_print_setup_dialog_new (void)
 
 	dialog = EPHY_DIALOG (g_object_new (EPHY_TYPE_DIALOG, NULL));
 
-	ephy_dialog_construct (dialog, setup_props,
-			       "print.glade", "print_setup_dialog");
+	ephy_dialog_construct (dialog,
+			       setup_props,
+			       ephy_file ("print.glade"),
+			       "print_setup_dialog",
+			       NULL);
 
 	ephy_dialog_add_enum (dialog, setup_props[PAPER_PROP].id,
 			      n_paper_format_enum, paper_format_enum);
