@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-dialog.c,v 1.27 2004/01/24 18:41:33 marco Exp $
+ *  $Id: ephy-dialog.c,v 1.31.4.2 2004/04/18 14:17:07 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,7 +44,9 @@ enum
 	PROP_0,
 	PROP_PARENT_WINDOW,
 	PROP_MODAL,
-	PROP_PERSIST_POSITION
+	PROP_PERSIST_POSITION,
+	PROP_DEFAULT_WIDTH,
+	PROP_DEFAULT_HEIGHT
 };
 
 typedef enum
@@ -86,6 +88,8 @@ struct EphyDialogPrivate
 	gboolean disposing;
 	gboolean initialized;
 	gboolean persist_position;
+	int default_width;
+	int default_height;
 };
 
 #define SPIN_DELAY 0.20
@@ -153,6 +157,10 @@ set_sensitivity (PropertyInfo *info, gboolean sensitive)
 			gtk_widget_set_sensitive (GTK_WIDGET (l->data), sensitive);
 		}
 	}
+	else if (info->widget_type == PT_EDITABLE)
+	{
+		gtk_editable_set_editable (GTK_EDITABLE (info->widget), sensitive);
+	}
 	else
 	{
 		gtk_widget_set_sensitive (info->widget, sensitive);
@@ -187,8 +195,6 @@ set_value_from_pref (PropertyInfo *info, GValue *value)
 			g_warning ("Unsupported value read from pref %s\n", info->pref);
 			break;
 	}
-
-	LOG ("id[%s], pref[%s] = %s", info->id, info->pref, g_strdup_value_contents (value))
 }
 
 static void
@@ -221,13 +227,14 @@ set_pref_from_value (PropertyInfo *info, GValue *value)
 	}
 }
 
-static void
+static gboolean
 set_value_from_editable (PropertyInfo *info, GValue *value)
 {
 	char *text;
+	gboolean retval = TRUE;
 	gboolean free_text = TRUE;
 
-	g_return_if_fail (GTK_IS_EDITABLE (info->widget));
+	g_return_val_if_fail (GTK_IS_EDITABLE (info->widget), FALSE);
 
 	text = gtk_editable_get_chars (GTK_EDITABLE (info->widget), 0, -1);
 
@@ -246,6 +253,7 @@ set_value_from_editable (PropertyInfo *info, GValue *value)
 			g_value_set_float (value, strtod (text, NULL));
 			break;
 		default:
+			retval = FALSE;
 			g_warning ("Unsupported value type for editable %s", info->id);
 			break;
 	}
@@ -254,43 +262,48 @@ set_value_from_editable (PropertyInfo *info, GValue *value)
 	{
 		g_free (text);
 	}
+
+	return retval;
 }
 
-static void
+static gboolean
 set_value_from_combobox (PropertyInfo *info, GValue *value)
 {
-	int index;
-
-	g_return_if_fail (GTK_IS_COMBO_BOX (info->widget));
-
-	index = gtk_combo_box_get_active (GTK_COMBO_BOX (info->widget));
-	g_return_if_fail (index >= 0);
+	g_return_val_if_fail (GTK_IS_COMBO_BOX (info->widget), FALSE);
 
 	if (info->data_col != -1)
 	{
 		GtkTreeModel *model;
 		GtkTreeIter iter;
 
-		model = gtk_combo_box_get_model (GTK_COMBO_BOX (info->widget));
-
-		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, index))
+		if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (info->widget), &iter))
 		{
+			model = gtk_combo_box_get_model (GTK_COMBO_BOX (info->widget));
 			gtk_tree_model_get_value (model, &iter, info->data_col, value);
-		}
-		else
-		{
-			g_warning ("Invalid index in combo model for %s\n", info->id);
+
+			return TRUE;
 		}
 	}
 	else if (info->data_type == G_TYPE_INT)
 	{
-		g_value_init (value, G_TYPE_INT);
-		g_value_set_int (value, index);
+		int index;
+
+		index = gtk_combo_box_get_active (GTK_COMBO_BOX (info->widget));
+
+		if (index >= 0)
+		{
+			g_value_init (value, G_TYPE_INT);
+			g_value_set_int (value, index);
+
+			return TRUE;
+		}
 	}
 	else
 	{
 		g_warning ("Unsupported data type for combo %s\n", info->id);
 	}
+
+	return FALSE;
 }
 
 static int
@@ -322,20 +335,22 @@ get_radio_button_active_index (GtkWidget *radiobutton)
 	return index = (length - 1) - i;
 }
 
-static void
+static gboolean
 set_value_from_radiobuttongroup (PropertyInfo *info, GValue *value)
 {
+	gboolean retval = TRUE;
 	int index;
 
-	g_return_if_fail (GTK_IS_RADIO_BUTTON (info->widget));
+	g_return_val_if_fail (GTK_IS_RADIO_BUTTON (info->widget), FALSE);
 
 	index = get_radio_button_active_index (info->widget);
-	g_return_if_fail (index >= 0);
+	g_return_val_if_fail (index >= 0, FALSE);
 
 	g_value_init (value, info->data_type);
 	if (info->data_type == G_TYPE_STRING)
 	{
-		g_return_if_fail (info->string_enum != NULL);
+		g_return_val_if_fail (info->string_enum != NULL, FALSE);
+		g_return_val_if_fail (g_list_nth_data (info->string_enum, index) != NULL, FALSE);
 
 		g_value_set_string (value, (char*) g_list_nth_data (info->string_enum, index));
 	}
@@ -345,17 +360,21 @@ set_value_from_radiobuttongroup (PropertyInfo *info, GValue *value)
 	}
 	else
 	{
+		retval = FALSE;
 		g_warning ("unsupported data type for radio button %s\n", info->id);
 	}
+
+	return retval;
 }
 
-static void
+static gboolean
 set_value_from_spin_button (PropertyInfo *info, GValue *value)
 {
+	gboolean retval = TRUE;
 	gdouble f;
 	gboolean is_int;
 
-	g_return_if_fail (GTK_IS_SPIN_BUTTON (info->widget));
+	g_return_val_if_fail (GTK_IS_SPIN_BUTTON (info->widget), FALSE);
 
 	f = gtk_spin_button_get_value (GTK_SPIN_BUTTON (info->widget));
 
@@ -372,16 +391,20 @@ set_value_from_spin_button (PropertyInfo *info, GValue *value)
 	}
 	else
 	{
+		retval = FALSE;
 		g_warning ("Unsupported data type for spin button %s\n", info->id);
 	}
+
+	return retval;
 }
 
-static void
+static gboolean
 set_value_from_togglebutton (PropertyInfo *info, GValue *value)
 {
+	gboolean retval = TRUE;
 	gboolean active;
 
-	g_return_if_fail (GTK_IS_TOGGLE_BUTTON (info->widget));
+	g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (info->widget), FALSE);
 
 	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (info->widget));
 
@@ -392,39 +415,47 @@ set_value_from_togglebutton (PropertyInfo *info, GValue *value)
 	}
 	else
 	{
+		retval = FALSE;
 		g_warning ("Unsupported data type for toggle button %s\n", info->id);
 	}
+
+	return retval;
 }
 
-static void
+static gboolean
 set_value_from_info (PropertyInfo *info, GValue *value)
 {
+	gboolean retval;
+
 	if (info->sane_state == FALSE)
 	{
-		g_warning ("id[%s] has insane state when trying to get value!\n", info->id);
+		return FALSE;
 	}
 
 	switch (info->widget_type)
 	{
 		case PT_SPINBUTTON:
-			set_value_from_spin_button (info, value);
+			retval = set_value_from_spin_button (info, value);
 			break;
 		case PT_RADIOBUTTON:
-			set_value_from_radiobuttongroup (info, value);
+			retval = set_value_from_radiobuttongroup (info, value);
 			break;
 		case PT_TOGGLEBUTTON:
-			set_value_from_togglebutton (info, value);
+			retval = set_value_from_togglebutton (info, value);
 			break;
 		case PT_EDITABLE:
-			set_value_from_editable (info, value);
+			retval = set_value_from_editable (info, value);
 			break;
 		case PT_COMBOBOX:
-			set_value_from_combobox (info, value);
+			retval = set_value_from_combobox (info, value);
 			break;
 		default:
+			retval = FALSE;
 			g_warning ("Unsupported widget type\n");
 			break;
 	}
+
+	return retval;
 }
 
 static void
@@ -524,8 +555,6 @@ compare_values (const GValue *a, const GValue *b)
 static void
 set_combo_box_from_value (PropertyInfo *info, const  GValue *value)
 {
-	int index = -1;
-
 	g_return_if_fail (GTK_IS_COMBO_BOX (info->widget));
 
 	if (info->data_col != -1)
@@ -542,45 +571,44 @@ set_combo_box_from_value (PropertyInfo *info, const  GValue *value)
 		{
 			gtk_tree_model_get_value (model, &iter, info->data_col, &data);
 			found = compare_values (&data, value);
+			g_value_unset (&data);
+
 			if (found) break;
 
-			g_value_unset (&data);
 			valid = gtk_tree_model_iter_next (model, &iter);
 		}
 
 		if (found)
 		{
-			GtkTreePath *path;
-			gint *indices;
-
-			path = gtk_tree_model_get_path (model, &iter);
-			indices = gtk_tree_path_get_indices (path);
-			index = indices[0];
-			gtk_tree_path_free (path);
+			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (info->widget), &iter);
 		}
+		else
+		{
+			gtk_combo_box_set_active (GTK_COMBO_BOX (info->widget), -1);
+
+			char *v = g_strdup_value_contents (value);
+			g_warning ("Value '%s' not found in model for combo %s\n", v, info->id);
+			g_free (v);
+		}
+
+		info->sane_state = found;
 	}
 	else if (info->data_type == G_TYPE_INT)
 	{
+		int index;
+
 		index = g_value_get_int (value);
+
+		info->sane_state = index >= 0;
+
+		g_return_if_fail (index >= -1);
+
+		gtk_combo_box_set_active (GTK_COMBO_BOX (info->widget), index);
 	}
 	else
 	{
 		g_warning ("Unsupported data type for combo box %s\n", info->id);
 	}
-
-	if (index < 0)
-	{
-		info->sane_state = FALSE;
-
-		g_return_if_fail (index >= 0);
-		return;
-	}
-
-	LOG ("index[%s] is %d", info->id, index)
-
-	info->sane_state = TRUE;
-
-	gtk_combo_box_set_active (GTK_COMBO_BOX (info->widget), index);
 }
 
 static void
@@ -705,14 +733,11 @@ set_pref_from_info_and_emit (PropertyInfo *info)
 {
 	GValue value = { 0, };
 
-	if (!info->sane_state)
+	if (!set_value_from_info (info, &value))
 	{
-		g_warning ("Not emitting/persisting insane state of id[%s]", info->id);
 		return;
 	}
 
-	set_value_from_info (info, &value);
-
 	g_signal_emit (info->dialog, signals[CHANGED], g_quark_from_string (info->id), &value);
 
 	if (info->apply_type == PT_AUTOAPPLY && info->pref != NULL)
@@ -812,8 +837,6 @@ connect_signals (gpointer key, PropertyInfo *info, EphyDialog *dialog)
 
 	g_return_if_fail (info->widget != NULL);
 
-	if (info->apply_type != PT_AUTOAPPLY) return;
-
 	switch (info->widget_type)
 	{
 		case PT_TOGGLEBUTTON:
@@ -846,7 +869,6 @@ connect_signals (gpointer key, PropertyInfo *info, EphyDialog *dialog)
 					  G_CALLBACK (changed_cb), info);
 			break;
 		case PT_UNKNOWN:
-			g_warning ("Unsupported widget type\n");
 			break;
 	}
 }
@@ -958,9 +980,11 @@ save_info (gpointer key, PropertyInfo *info, EphyDialog *dialog)
 		return;
 	}
 
-	set_value_from_info (info, &value);
-	set_pref_from_value (info, &value);
-	g_value_unset (&value);
+	if (set_value_from_info (info, &value))
+	{
+		set_pref_from_value (info, &value);
+		g_value_unset (&value);
+	}
 }
 
 static void
@@ -978,7 +1002,9 @@ setup_default_size (EphyDialog *dialog)
 		}
 
 		ephy_state_add_window (dialog->priv->dialog,
-				       dialog->priv->name, -1, -1,
+				       dialog->priv->name,
+				       dialog->priv->default_width,
+				       dialog->priv->default_height,
 				       flags);
 
 		dialog->priv->has_default_size = TRUE;
@@ -1175,50 +1201,11 @@ ephy_dialog_hide (EphyDialog *dialog)
 	gtk_widget_hide (dialog->priv->dialog);
 }
 
-#if 0
-static void
-run_response_cb (GtkWidget *dialog,
-		 int response,
-		 int *result)
-{
-	*result = response;
-
-	gtk_grab_remove (dialog);
-	LOG ("run_response_cb: leaving gtk level %d", gtk_main_level())
-	gtk_main_quit();
-}
-#endif
-
 int
 ephy_dialog_run (EphyDialog *dialog)
 {
 	ephy_dialog_show (dialog);
 
-#if 0
-	if (dialog->priv->parent != NULL && dialog->priv->modal == FALSE)
-	{
-		GtkWindowGroup *group;
-		int response = 0;
-
-		group = GTK_WINDOW (dialog->priv->parent)->group;
-		if (group == NULL)
-		{
-			group = gtk_window_group_new ();
-			gtk_window_group_add_window (group, GTK_WINDOW (dialog->priv->parent));
-			g_object_unref (group);
-		}
-
-		gtk_window_group_add_window (group, GTK_WINDOW (dialog->priv->dialog));
-		g_signal_connect(dialog->priv->dialog, "response",
-				 G_CALLBACK (run_response_cb), &response);
-		gtk_grab_add (dialog->priv->dialog);
-		LOG ("ephy_dialog_run before main(): level %d", gtk_main_level())
-		gtk_main ();
-		LOG ("ephy_dialog_run after main(): level %d", gtk_main_level())
-
-		return response;
-	}
-#endif
 	return gtk_dialog_run (GTK_DIALOG (dialog->priv->dialog));
 }
 
@@ -1234,7 +1221,7 @@ ephy_dialog_get_control (EphyDialog *dialog,
 	return info->widget;
 }
 
-void
+gboolean
 ephy_dialog_get_value (EphyDialog *dialog,
 		       const char *property_id,
 		       GValue *value)
@@ -1242,9 +1229,9 @@ ephy_dialog_get_value (EphyDialog *dialog,
 	PropertyInfo *info;
 
 	info = lookup_info (dialog, property_id);
-	g_return_if_fail (info != NULL);
+	g_return_val_if_fail (info != NULL, FALSE);
 
-	set_value_from_info (info, value);
+	return set_value_from_info (info, value);
 }
 
 void
@@ -1286,6 +1273,8 @@ ephy_dialog_init (EphyDialog *dialog)
 	dialog->priv->has_default_size = FALSE;
 	dialog->priv->disposing = FALSE;
 	dialog->priv->persist_position = FALSE;
+	dialog->priv->default_width = -1;
+	dialog->priv->default_height = -1;
 
 	dialog->priv->props = g_hash_table_new_full 
 		(g_str_hash, g_str_equal, NULL, (GDestroyNotify) free_prop_info);
@@ -1316,7 +1305,15 @@ ephy_dialog_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
+GtkWidget *
+ephy_dialog_get_parent (EphyDialog *dialog)
+{
+	g_return_val_if_fail (EPHY_IS_DIALOG (dialog), NULL);
+
+	return dialog->priv->parent;
+}
+
+void
 ephy_dialog_set_parent (EphyDialog *dialog,
 			GtkWidget *parent)
 {
@@ -1344,6 +1341,12 @@ ephy_dialog_set_property (GObject *object,
 		case PROP_PERSIST_POSITION:
 			dialog->priv->persist_position = g_value_get_boolean (value);
 			break;
+		case PROP_DEFAULT_WIDTH:
+			dialog->priv->default_width = g_value_get_int (value);
+			break;
+		case PROP_DEFAULT_HEIGHT:
+			dialog->priv->default_height = g_value_get_int (value);
+			break;
 	}
 }
 
@@ -1366,6 +1369,12 @@ ephy_dialog_get_property (GObject *object,
 		case PROP_PERSIST_POSITION:
 			g_value_set_boolean (value, dialog->priv->persist_position);
 			break;
+		case PROP_DEFAULT_WIDTH:
+			g_value_set_int (value, dialog->priv->default_width);
+			break;
+		case PROP_DEFAULT_HEIGHT:
+			g_value_set_int (value, dialog->priv->default_height);
+			break;
 	}
 }
 
@@ -1417,7 +1426,30 @@ ephy_dialog_class_init (EphyDialogClass *klass)
 							       "Persist position",
 							       "Persist dialog position",
 							       FALSE,
-							       G_PARAM_READWRITE));
+							       G_PARAM_READWRITE |
+							       G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_DEFAULT_WIDTH,
+					 g_param_spec_int ("default-width",
+							   "Default width",
+							   "Default dialog width",
+							   -1,
+							   G_MAXINT,
+							   -1,
+							   G_PARAM_READWRITE |
+							   G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_DEFAULT_HEIGHT,
+					 g_param_spec_int ("default-height",
+							   "Default height",
+							   "Default dialog height",
+							   -1,
+							   G_MAXINT,
+							   -1,
+							   G_PARAM_READWRITE |
+							   G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (object_class, sizeof (EphyDialogPrivate));
 }
