@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-import.c,v 1.19 2004/01/09 20:49:11 clahey Exp $
+ *  $Id: ephy-bookmarks-import.c,v 1.26.4.1 2004/03/22 09:17:33 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -100,11 +100,12 @@ ephy_bookmarks_import (EphyBookmarks *bookmarks,
 	{
 		return ephy_bookmarks_import_mozilla (bookmarks, filename);
 	}
-	else if (strcmp (type, "application/xbel") == 0)
+	else if (strcmp (type, "application/x-xbel") == 0)
 	{
 		return ephy_bookmarks_import_xbel (bookmarks, filename);
 	}
-	else if (strstr (filename, MOZILLA_BOOKMARKS_DIR) != NULL)
+	else if (strstr (filename, MOZILLA_BOOKMARKS_DIR) != NULL ||
+                 strstr (filename, FIREBIRD_BOOKMARKS_DIR) != NULL)
 	{
 		return ephy_bookmarks_import_mozilla (bookmarks, filename);
 	}
@@ -121,6 +122,9 @@ ephy_bookmarks_import (EphyBookmarks *bookmarks,
 
 typedef enum
 {
+	STATE_START,
+	STATE_STOP,
+	STATE_XBEL,
 	STATE_FOLDER,
 	STATE_BOOKMARK,
 	STATE_TITLE,
@@ -130,8 +134,8 @@ typedef enum
 	STATE_SMARTURL
 } EphyXBELImporterState;
 
-static EphyNode *
-xbel_parse_bookmark (EphyBookmarks *eb, xmlTextReaderPtr reader)
+static int
+xbel_parse_bookmark (EphyBookmarks *eb, xmlTextReaderPtr reader, EphyNode **ret_node)
 {
 	EphyXBELImporterState state = STATE_BOOKMARK;
 	EphyNode *node;
@@ -141,11 +145,11 @@ xbel_parse_bookmark (EphyBookmarks *eb, xmlTextReaderPtr reader)
 
 	while (ret == 1)
 	{
-		xmlChar *tag;
+		const xmlChar *tag;
 		xmlReaderTypes type;
 
-		tag = xmlTextReaderName (reader);
-		g_return_val_if_fail (tag != NULL, NULL);
+		tag = xmlTextReaderConstName (reader);
+		g_return_val_if_fail (tag != NULL, ret);
 
 		type = xmlTextReaderNodeType (reader);
 
@@ -234,13 +238,14 @@ xbel_parse_bookmark (EphyBookmarks *eb, xmlTextReaderPtr reader)
 			}
 		}
 
-		xmlFree (tag);
-
 		/* next one, please */
 		ret = xmlTextReaderRead (reader);
 	}
 
-	g_return_val_if_fail (address != NULL, NULL);
+	if (address == NULL)
+	{
+		return ret;
+	}
 
 	if (title == NULL)
 	{
@@ -257,11 +262,13 @@ xbel_parse_bookmark (EphyBookmarks *eb, xmlTextReaderPtr reader)
 	xmlFree (title);
 	xmlFree (address);
 
-	return node;
+	*ret_node = node;
+
+	return ret;
 }
 
-static GList *
-xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader)
+static int
+xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader, GList **ret_list)
 {
 	EphyXBELImporterState state = STATE_FOLDER;
 	EphyNode *keyword;
@@ -273,10 +280,10 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader)
 
 	while (ret == 1)
 	{
-		xmlChar *tag;
+		const xmlChar *tag;
 		xmlReaderTypes type;
 
-		tag = xmlTextReaderName (reader);
+		tag = xmlTextReaderConstName (reader);
 		type = xmlTextReaderNodeType (reader);
 
 		if (tag == NULL)
@@ -296,25 +303,29 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader)
 		}
 		else if (xmlStrEqual (tag, "bookmark") && type == 1 && state == STATE_FOLDER)
 		{
-			EphyNode *node;
+			EphyNode *node = NULL;
 
-			node = xbel_parse_bookmark (eb, reader);
+			ret = xbel_parse_bookmark (eb, reader, &node);
 
 			if (EPHY_IS_NODE (node))
 			{
 				list = g_list_prepend (list, node);
 			}
+
+			if (ret != 1) break;
 		}
-		else if ((xmlStrEqual (tag, "folder") || xmlStrEqual (tag, "xbel"))
+		else if ((xmlStrEqual (tag, "folder"))
 			&& state == STATE_FOLDER)
 		{
 			if (type == XML_READER_TYPE_ELEMENT)
 			{
-				GList *sublist;
+				GList *sublist = NULL;
 
-				sublist = xbel_parse_folder (eb, reader);
+				ret = xbel_parse_folder (eb, reader, &sublist);
 
 				list = g_list_concat (list, sublist);
+				
+				if (ret != 1) break;
 			}
 			else if (type == XML_READER_TYPE_END_ELEMENT)
 			{
@@ -361,16 +372,15 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader)
 			/* eat it */
 		}
 
-		xmlFree (tag);
-
 		/* next one, please */
 		ret = xmlTextReaderRead (reader);
 	}
 
 	/* tag all bookmarks in the list with keyword %title */
-	if (title == NULL)
+	if (title == NULL || title[0] == '\0')
 	{
-		title = xmlStrdup (_("Untitled"));
+		*ret_list = list;
+		return ret;
 	}
 
 	keyword = ephy_bookmarks_find_keyword (eb, title, FALSE);
@@ -382,7 +392,11 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader)
 
 	xmlFree (title);
 
-	if (keyword == NULL) return list;
+	if (keyword == NULL)
+	{
+		*ret_list = list;
+		return ret;
+	}
 
 	for (l = list; l != NULL; l = l->next)
 	{
@@ -391,7 +405,102 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader)
 		ephy_bookmarks_set_keyword (eb, keyword, node);
 	}
 
-	return list;
+	*ret_list = list;
+
+	return ret;
+}
+
+static int
+xbel_parse_xbel (EphyBookmarks *eb, xmlTextReaderPtr reader)
+{
+	EphyXBELImporterState state = STATE_XBEL;
+	int ret;
+
+	ret = xmlTextReaderRead (reader);
+
+	while (ret == 1 && state != STATE_STOP)
+	{
+		const xmlChar *tag;
+		xmlReaderTypes type;
+
+		tag = xmlTextReaderConstName (reader);
+		type = xmlTextReaderNodeType (reader);
+
+		if (tag == NULL)
+		{
+			/* shouldn't happen but does anyway :( */
+		}
+		else if (xmlStrEqual (tag, "bookmark") && type == XML_READER_TYPE_ELEMENT
+			 && state == STATE_FOLDER)
+		{
+			EphyNode *node = NULL;
+
+			/* this will eat the </bookmark> too */
+			ret = xbel_parse_bookmark (eb, reader, &node);
+
+			if (ret != 1) break;
+		}
+		else if (xmlStrEqual (tag, "folder") && type == XML_READER_TYPE_ELEMENT
+			 && state == STATE_XBEL)
+		{
+			GList *list = NULL;
+
+			/* this will eat the </folder> too */
+			ret = xbel_parse_folder (eb, reader, &list);
+
+			g_list_free (list);
+
+			if (ret != 1) break;
+		}
+		else if ((xmlStrEqual (tag, "xbel")) && type == XML_READER_TYPE_ELEMENT
+			 && state == STATE_START)
+		{
+			state = STATE_XBEL;
+		}
+		else if ((xmlStrEqual (tag, "xbel")) && type == XML_READER_TYPE_END_ELEMENT
+			 && state == STATE_XBEL)
+		{
+			state = STATE_STOP;
+		}
+		else if (xmlStrEqual (tag, "title"))
+		{
+			if (type == XML_READER_TYPE_ELEMENT && state == STATE_XBEL)
+			{
+				state = STATE_TITLE;
+			}
+			else if (type == XML_READER_TYPE_END_ELEMENT && state == STATE_TITLE)
+			{
+				state = STATE_XBEL;
+			}
+		}
+		else if (xmlStrEqual (tag, "info"))
+		{
+			if (type == XML_READER_TYPE_ELEMENT && state == STATE_XBEL)
+			{
+				state = STATE_INFO;
+			}
+			else if (type == XML_READER_TYPE_END_ELEMENT && state == STATE_INFO)
+			{
+				state = STATE_XBEL;
+			}
+		}
+		else if (xmlStrEqual (tag, "desc"))
+		{
+			if (type == XML_READER_TYPE_ELEMENT && state == STATE_XBEL)
+			{
+				state = STATE_DESC;
+			}
+			else if (type == XML_READER_TYPE_END_ELEMENT && state == STATE_DESC)
+			{
+				state = STATE_XBEL;
+			}
+		}
+
+		/* next one, please */
+		ret = xmlTextReaderRead (reader);
+	}
+
+	return ret;
 }
 
 /* Mozilla/Netscape import */
@@ -490,6 +599,8 @@ ns_get_bookmark_item (FILE *f, GString *name, GString *url)
  * More info : http://www.w3.org/TR/html4/charset.html#h-5.3.2
  * NOTE : We don't support &#D or &#xH.
  * Patch courtesy of Almer S. Tigelaar <almer1@dds.nl>
+ *
+ * NOTE: The returned string must be freed.
  */
 static char *
 ns_parse_bookmark_item (GString *string)
@@ -557,7 +668,7 @@ ns_parse_bookmark_item (GString *string)
 			for (i = jump - 1; i > 0; i--)
 			{
 				iterator++;
-				if (iterator == NULL)
+				if (*iterator == '\0')
 					break;
 			}
 		}
@@ -577,7 +688,7 @@ ephy_bookmarks_import_mozilla (EphyBookmarks *bookmarks,
 {
 	FILE *bf;  /* bookmark file */
 	GString *name;
-	gchar *parsedname;
+	char *parsedname;
 	GString *url = g_string_new (NULL);
 	GList *folders = NULL, *l;
 
@@ -599,7 +710,7 @@ ephy_bookmarks_import_mozilla (EphyBookmarks *bookmarks,
 		switch (t)
 		{
 		case NS_FOLDER:
-			folders = g_list_prepend (folders, g_strdup (name->str));
+			folders = g_list_prepend (folders, ns_parse_bookmark_item (name));
 			break;
 		case NS_FOLDER_END:
 			if (folders)
@@ -636,6 +747,9 @@ ephy_bookmarks_import_mozilla (EphyBookmarks *bookmarks,
 					ephy_bookmarks_set_keyword (bookmarks, keyword, node);
 				}
 			}
+
+			g_free (parsedname);
+
 			break;
 		default:
 			break;
@@ -653,7 +767,7 @@ ephy_bookmarks_import_xbel (EphyBookmarks *bookmarks,
 			    const char *filename)
 {
 	xmlTextReaderPtr reader;
-	GList *list;
+	int ret;
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING)) return FALSE;
 
@@ -663,14 +777,16 @@ ephy_bookmarks_import_xbel (EphyBookmarks *bookmarks,
 	}
 	
 	reader = xmlNewTextReaderFilename (filename);
-	g_return_val_if_fail (reader != NULL, FALSE);
+	if (reader == NULL)
+	{
+		return FALSE;
+	}
 
-	list = xbel_parse_folder (bookmarks, reader);
+	ret = xbel_parse_xbel (bookmarks, reader);
 
-	g_list_free (list);
 	xmlFreeTextReader (reader);
 
-	return TRUE;
+	return ret >= 0 ? TRUE : FALSE;
 }
 
 #define OLD_RDF_TEMPORARY_HACK
@@ -804,7 +920,15 @@ ephy_bookmarks_import_rdf (EphyBookmarks *bookmarks,
 		return FALSE;
 
 	doc = xmlParseFile (filename);
-	g_assert (doc != NULL);
+	if (doc == NULL)
+	{
+		/* FIXME: maybe put up a warning dialogue here, because this
+		 * is a severe dataloss?
+		 */
+		g_warning ("Failed to re-import the bookmarks. All bookmarks lost!\n");
+		return FALSE;
+	}
+
 	root = xmlDocGetRootElement (doc);
 
 	child = root->children;
