@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-main.c,v 1.23 2004/02/12 16:47:41 chpe Exp $
+ *  $Id: ephy-main.c,v 1.24 2004/03/05 23:38:10 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,6 +30,7 @@
 #include "eel-gconf-extensions.h"
 
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-app-helper.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmessagedialog.h>
 #include <libgnome/gnome-program.h>
@@ -193,6 +194,7 @@ main (int argc, char *argv[])
 	eel_gconf_monitor_remove ("/apps/epiphany/general");
 	eel_gconf_monitor_remove ("/apps/epiphany/lockdown");
 	eel_gconf_monitor_remove ("/desktop/gnome/lockdown");
+	gnome_accelerators_sync ();
 	ephy_state_save ();
 	ephy_file_helpers_shutdown ();
 	gnome_vfs_shutdown ();
