@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-session.c,v 1.8 2004/01/25 13:54:41 chpe Exp $
+ *  $Id: ephy-session.c,v 1.14 2004/03/02 12:42:34 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -54,6 +54,7 @@
 struct EphySessionPrivate
 {
 	GList *windows;
+	GtkWidget *resume_dialog;
 };
 
 #define BOOKMARKS_EDITOR_ID	"BookmarksEditor"
@@ -61,7 +62,7 @@ struct EphySessionPrivate
 #define SESSION_CRASHED		"type:session_crashed"
 
 static void ephy_session_class_init	(EphySessionClass *klass);
-static void ephy_session_iface_init	(EphyExtensionClass *iface);
+static void ephy_session_iface_init	(EphyExtensionIface *iface);
 static void ephy_session_init		(EphySession *session);
 
 static GObjectClass *parent_class = NULL;
@@ -151,22 +152,22 @@ net_stop_cb (EphyEmbed *embed,
 
 static void
 tab_added_cb (GtkWidget *notebook,
-	      GtkWidget *child,
+	      EphyTab *tab,
 	      EphySession *session)
 {
-	g_signal_connect (child, "net_stop",
+	g_signal_connect (ephy_tab_get_embed (tab), "net_stop",
 			  G_CALLBACK (net_stop_cb), session);
 }
 
 static void
 tab_removed_cb (GtkWidget *notebook,
-		GtkWidget *child,
+		EphyTab *tab,
 		EphySession *session)
 {
 	ephy_session_save (session, SESSION_CRASHED);
 
 	g_signal_handlers_disconnect_by_func
-		(child, G_CALLBACK (net_stop_cb), session);
+		(ephy_tab_get_embed (tab), G_CALLBACK (net_stop_cb), session);
 }
 
 static void
@@ -185,7 +186,7 @@ impl_attach_window (EphyExtension *extension,
 
 	LOG ("impl_attach_window")
 
-	session->priv->windows = g_list_prepend (session->priv->windows, window);
+	session->priv->windows = g_list_append (session->priv->windows, window);
 	ephy_session_save (session, SESSION_CRASHED);
 
 	notebook = ephy_window_get_notebook (window);
@@ -238,6 +239,7 @@ ephy_session_init (EphySession *session)
 	LOG ("EphySession initialising")
 
 	session->priv->windows = NULL;
+	session->priv->resume_dialog = NULL;
 
 	ensure_session_directory ();
 }
@@ -265,7 +267,7 @@ ephy_session_finalize (GObject *object)
 }
 
 static void
-ephy_session_iface_init (EphyExtensionClass *iface)
+ephy_session_iface_init (EphyExtensionIface *iface)
 {
 	iface->attach_window = impl_attach_window;
 	iface->detach_window = impl_detach_window;
@@ -301,6 +303,7 @@ offer_to_resume (EphySession *session)
 		 _("_Don't Recover"), GTK_RESPONSE_CANCEL,
 		 _("_Recover"), GTK_RESPONSE_OK,
 		 NULL);
+	session->priv->resume_dialog = dialog;
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
@@ -325,6 +328,7 @@ offer_to_resume (EphySession *session)
 			    TRUE, TRUE, 0);
 
 	label = gtk_label_new (NULL);
+	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_widget_show (label);
@@ -335,6 +339,7 @@ offer_to_resume (EphySession *session)
 	g_free (str);
 
 	label = gtk_label_new (_("You can recover the opened tabs and windows."));
+	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (vbox), label,
@@ -344,6 +349,8 @@ offer_to_resume (EphySession *session)
 
 	gtk_widget_destroy (dialog);
 
+	session->priv->resume_dialog = NULL;
+
 	return (response == GTK_RESPONSE_OK);
 }
 
@@ -353,7 +360,9 @@ offer_to_resume (EphySession *session)
  *
  * Resume a crashed session when necessary (interactive)
  *
- * Return value: TRUE if at least a window has been opened
+ * Return value: TRUE if handled; windows have actually
+ * been opened or the dialog from a previous instance
+ * has been re-presented to the user.
  **/
 gboolean
 ephy_session_autoresume (EphySession *session)
@@ -361,9 +370,15 @@ ephy_session_autoresume (EphySession *session)
 	char *saved_session;
 	gboolean retval = FALSE;
 
+	LOG ("ephy_session_autoresume")
+
 	if (session->priv->windows != NULL) return FALSE;
 
-	LOG ("ephy_session_autoresume")
+	if (session->priv->resume_dialog)
+	{
+		gtk_window_present (GTK_WINDOW (session->priv->resume_dialog));
+		return TRUE;
+	}
 
 	saved_session = get_session_filename (SESSION_CRASHED);
 
@@ -522,6 +537,12 @@ ephy_session_save (EphySession *session,
 		return FALSE;
 	}
 
+	ret = xmlTextWriterSetIndent (writer, 1);
+	if (ret < 0) goto out;
+
+	ret = xmlTextWriterSetIndentString (writer, "  ");
+	if (ret < 0) goto out;
+
 	START_PROFILER ("Saving session")
 
 	ret = xmlTextWriterStartDocument (writer, "1.0", NULL, NULL);
@@ -718,7 +739,7 @@ ephy_session_add_window (EphySession *session,
 {
 	LOG ("ephy_session_add_window")
 
-	session->priv->windows = g_list_prepend (session->priv->windows, window);
+	session->priv->windows = g_list_append (session->priv->windows, window);
 
 	ephy_session_save (session, SESSION_CRASHED);
 }
