@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.95 2004/03/04 23:09:02 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.95.4.1 2004/03/22 09:21:30 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,6 +60,8 @@
 
 #define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
 
+#define CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME  "/apps/epiphany/lockdown/disable_javascript_chrome"
+
 struct EphyTabPrivate
 {
 	EphyWindow *window;
@@ -901,13 +903,19 @@ ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
 
 static void
 ephy_tab_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
-			EmbedChromeMask chromemask, EphyTab *tab)
+			EphyEmbedChrome chromemask, EphyTab *tab)
 {
 	EphyTab *new_tab;
 	EphyWindow *window;
 
-	window = ephy_window_new ();
-	ephy_window_request_chrome (window, chromemask);
+	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME))
+	{
+		window = ephy_window_new ();
+	}
+	else
+	{
+		window = ephy_window_new_with_chrome (chromemask);
+	}
 
 	new_tab = ephy_tab_new ();
 	gtk_widget_show (GTK_WIDGET (new_tab));
